from __future__ import unicode_literals

from pyseeyou import format, format_tree
from pyseeyou.grammar import ICUMessageFormat


def test_format():
    select = format(
        '{WHO, select, male {He} female {She} other {They}}.',
        {'WHO': 'male'}, 'en')

    assert select == 'He.'

    plural = format(
        '{NUM_TICKETS, plural, one {1 ticket} other {# tickets} =42 {42 ticketerinos}}.',
        {'NUM_TICKETS': 1}, 'en')

    assert plural == '1 ticket.'

    select_plural = format(
        '{GENDER, select, male {He wants} female {She wants} other {They want}} {NUM_TICKETS, plural, one {1 ticket} other {# tickets} =42 {42 ticketerinos}}.',
        {'GENDER': 'male', 'NUM_TICKETS': 5}, 'en')

    assert select_plural == 'He wants 5 tickets.'

    select_plural_offset = format(
        '{GENDER, select, male {He wants} female {She wants} other {They want}} {NUM_TICKETS, plural, offset:2 one {1 ticket} other {# tickets} =42 {42 ticketerinos}}.',
        {'GENDER': 'female', 'NUM_TICKETS': '44'}, 'en')

    assert select_plural_offset == 'She wants 42 ticketerinos.'


def test_format_tree():
    ast = ICUMessageFormat.parse(
        '{WHO, select, male {He} female {She} other {They}}.')

    select = format_tree(ast, {'WHO': 'male'}, 'en')
    assert select == 'He.'

    select = format_tree(ast, {'WHO': 'female'}, 'en')
    assert select == 'She.'

    select = format_tree(ast, {'WHO': 'other'}, 'en')
    assert select == 'They.'

    select = format_tree(ast, {}, 'en')
    assert select == 'They.'

    ast = ICUMessageFormat.parse(
        '{NUM_TICKETS, plural, offset:1 one {1 ticket} other {# tickets} =42 {42 ticketerinos}}.')

    plural = format_tree(ast, {'NUM_TICKETS': 1}, 'en')
    assert plural == '0 tickets.'

    plural = format_tree(ast, {'NUM_TICKETS': '1'}, 'en')
    assert plural == '0 tickets.'

    plural = format_tree(ast, {'NUM_TICKETS': '2'}, 'en')
    assert plural == '1 ticket.'

    plural = format_tree(ast, {'NUM_TICKETS': 2.0}, 'en')
    assert plural == '1.0 tickets.'

    plural = format_tree(ast, {'NUM_TICKETS': 5}, 'en')
    assert plural == '4 tickets.'

    plural = format_tree(ast, {'NUM_TICKETS': 43}, 'en')
    assert plural == '42 ticketerinos.'



# coding=utf-8
from __future__ import unicode_literals
import pytest

from pyseeyou.grammar import ICUMessageFormat
from pyseeyou.node_visitor import ICUNodeVisitor


def test_process_select_statement():
    msg = ICUMessageFormat.parse(
        '''{WHO, select, male {He} female {She} other {They}}.''')

    i = ICUNodeVisitor({'WHO': 'male'})
    assert i.visit(msg) == 'He.'

    i = ICUNodeVisitor({'WHO': 'female'})
    assert i.visit(msg) == 'She.'

    i = ICUNodeVisitor({'WHO:': 'other'})
    assert i.visit(msg) == 'They.'


def test_use_other_statement_if_no_select_arg():
    msg = ICUMessageFormat.parse(
        '''{WHO, select, male {He} female {She} other {They}}.''')

    i = ICUNodeVisitor({})
    assert i.visit(msg) == 'They.'


def test_process_plural_statement():
    msg = ICUMessageFormat.parse(
        '''{NUM_TICKETS, plural, one {1 ticket} other {# tickets} =42 {42 ticketerinos}}.''')

    i = ICUNodeVisitor({'NUM_TICKETS': 1})
    assert i.visit(msg) == '1 ticket.'

    i = ICUNodeVisitor({'NUM_TICKETS': 2})
    assert i.visit(msg) == '2 tickets.'

    i = ICUNodeVisitor({'NUM_TICKETS': 42})
    assert i.visit(msg) == '42 ticketerinos.'


def test_plural_statement_with_offset():
    msg = ICUMessageFormat.parse(
        '''{NUM_TICKETS, plural, offset:3 one {1 ticket} other {# tickets} =42 {42 ticketerinos}}.''')

    i = ICUNodeVisitor({'NUM_TICKETS': 2})
    assert i.visit(msg) == '0 tickets.'

    i = ICUNodeVisitor({'NUM_TICKETS': 4})
    assert i.visit(msg) == '1 ticket.'

    i = ICUNodeVisitor({'NUM_TICKETS': 45})
    assert i.visit(msg) == '42 ticketerinos.'


def test_plural_dict_uses_string():
    msg = ICUMessageFormat.parse(
        '''{NUM_TICKETS, plural, one {1 ticket} other {# tickets} =42 {42 ticketerinos}}.''')

    i = ICUNodeVisitor({'NUM_TICKETS': '1'})
    assert i.visit(msg) == '1 ticket.'

    i = ICUNodeVisitor({'NUM_TICKETS': '2'})
    assert i.visit(msg) == '2 tickets.'

    i = ICUNodeVisitor({'NUM_TICKETS': '42'})
    assert i.visit(msg) == '42 ticketerinos.'


def test_plural_dict_with_decimals():
    msg = ICUMessageFormat.parse(
        '''{NUM_TICKETS, plural, one {1 ticket} other {# tickets} =42.5 {42 ticketerinos}}.''')

    i = ICUNodeVisitor({'NUM_TICKETS': '1'})
    assert i.visit(msg) == '1 ticket.'

    i = ICUNodeVisitor({'NUM_TICKETS': '1.0'})
    assert i.visit(msg) == '1.0 tickets.'

    i = ICUNodeVisitor({'NUM_TICKETS': '42'})
    assert i.visit(msg) == '42 tickets.'

    i = ICUNodeVisitor({'NUM_TICKETS': '42.5'})
    assert i.visit(msg) == '42 ticketerinos.'


def test_msg_with_basic_replace():
    msg = ICUMessageFormat.parse(
        '''I {GREETING} this finds you well.''')

    i = ICUNodeVisitor({'GREETING': 'hope'})
    assert i.visit(msg) == 'I hope this finds you well.'

    i = ICUNodeVisitor({'GREETING': 'wish'})
    assert i.visit(msg) == 'I wish this finds you well.'

    i = ICUNodeVisitor({'GREETING': ''})
    assert i.visit(msg) == 'I  this finds you well.'


def test_msg_with_unicode_chars():
    msg = ICUMessageFormat.parse(
        '''{SYMBOL, select, snowman {☃} sun {☉} other {☹}}''')

    i = ICUNodeVisitor({'SYMBOL': 'snowman'})
    assert i.visit(msg) == '☃'

    i = ICUNodeVisitor({'SYMBOL': 'sun'})
    assert i.visit(msg) == '☉'

    i = ICUNodeVisitor({})
    assert i.visit(msg) == '☹'



from pyseeyou.grammar import ICUMessageFormat
from pyseeyou.node_visitor import ICUNodeVisitor


def format(msg, values, lang):
    '''
    :param msg: String, The message to parse
    :param values: Dict, The values to use in forming the final message
    :param lang: String, The language code for the language to use to decide on
                 pluralisation rules
    '''

    ast = ICUMessageFormat.parse(msg)

    return format_tree(ast, values, lang)


def format_tree(ast, values, lang):
    '''
    :param ast: parsimonious.nodes.Node, Formed tree of the message to parse
    :param values: Dict, The values to use in forming the final message
    :param lang: String, The language code for the language to use to decide on
                 pluralisation rules
    '''
    i = ICUNodeVisitor(values, lang)

    return i.visit(ast)



from setuptools import find_packages, setup

setup(
    name='pyseeyou',
    version='0.4.0',
    description='A Python Parser for the ICU MessageFormat.',
    author='Siame Rafiq',
    author_email='mail@sia.me.uk',
    packages=find_packages(exclude=['tests']),
    install_requires=['parsimonious', 'toolz', 'future'],
    tests_require=['pytest'],
    url='https://github.com/rolepoint/pyseeyou',
    download_url='https://github.com/rolepoint/pyseeyou/archive/v0.4.0.tar.gz',
    keywords='python i18n messageformat python3 python2')
